{$r Plasma.rsrc}
{$appl MSpl} // CreatorID
Program Plasma(cmd);
{$i PalmAPI.pas}
{$i Double.pas}

//{$define COMPARE}
// Remove the comment from the line above to make this program comparable with
// the ARM version. Be warned that it will be extremely slow, so it will take
// a while to exit after selecting the home button on yoour palm...

Var
  Cmd: UInt16;

Procedure WinSetForeColorRGB(var NewRgbP:RGBColorType; var PrevRgbP:RGBColorType); inline(SYSTRAP,$A3E8);

Const
  {$ifdef COMPARE}
  NumSqr=159;
  SizeSqr=1;
  IncSqr=1;
  I1=1;
  I2=2;
  {$endif}
  {$ifndef COMPARE}
  NumSqr=39;
  SizeSqr=4;
  IncSqr=4;
  I1=7;
  I2=8;
  {$endif}

Type
  //This type is for storing a color palette
  TPalet = Array[0..255] of RGBColorType;

Var
  //This will contain the colors
  Palet: Array[0..255] of RGBColorType;
  //This will contain a precalculated COS function
  CosTbl: Array[0..255] of UInt8;
  a1,a2,a3,a4,b1,b2,b3,b4: UInt8;

//Will draw a colored box
Procedure DrawCell(X,Y: Integer; col: RGBColorType);
Var
  Rec: RectangleType;
  old: RGBColorType;
begin
  WinSetForeColorRGB(col,old);
  With Rec do begin
    topLeft.X := X;
    topLeft.y := Y;
    extent.x  := SizeSqr;
    extent.y  := SizeSqr;
    WinDrawRectangle(Rec,0)
  end;
end;

//Draw next screen
//This is a simple plasma routine...
Procedure DrawGameBoard;
Var
  x,y,x1,y1: Integer;
  b: byte;
begin
  a1:=b1;
  a2:=b2;
  y1:=0;
  for y:=0 to NumSqr do
    begin
      a3:=b3;
      a4:=b4;
      x1:=0;
      for x:=0 to NumSqr do
        begin
          b:=CosTbl[a1]+CosTbl[a2]+CosTbl[a3]+CosTbl[a4];
          DrawCell(x1,y1,Palet[b]);
          x1:=x1+IncSqr;
          a3:=a3+I1;
          a4:=a4+I2;
        end;
      y1:=y1+IncSqr;
      a1:=a1+I2;
      a2:=a2+I1;
    end;
  b1:=b1+2;
  b2:=b2-3;
  b3:=b3-2;
  b4:=b4+4;
end;

//Initialize the color palette in an array
Procedure InitColors;
Var
  RGB: RGBColorType;
  i: UInt8;
begin
  for i:=0 to 63 do
    begin
      Palet[i].R:=0;
      Palet[i].G:=i*4;
      Palet[i].B:=255;
      Palet[i+64].R:=i*4;
      Palet[i+64].G:=255;
      Palet[i+64].B:=255-i*4;
      Palet[i+128].R:=255;
      Palet[i+128].G:=255-i*4;
      Palet[i+128].B:=0;
      Palet[i+192].R:=255-i*4;
      Palet[i+192].G:=0;
      Palet[i+192].B:=i*4;
    end;
end;

//Initialize the screen
Procedure InitScr;
Var
  WidthP,HeightP,DepthP: UInt32;
  EnableColorP: Boolean;
  Error: Err;
begin
  WidthP:= 160;
  HeightP:= 160;
  DepthP:=16;
  EnableColorP:= True;
  Error:=WinScreenMode(winScreenModeSet,WidthP,HeightP,DepthP,EnableColorP);
  InitColors;
end;

//Main procedure
//Nothing complex: draws the screen and looks for the stop event to exit
Procedure Main;
Var
  Event: EventType;
begin
  Repeat
    DrawGameBoard;
    EvtGetEvent(Event, 0);
    SysHandleEvent(Event)
  Until (Event.eType=appStopEvent);
end;

//This initialize the variables used by the program
Function Initialize: Boolean;
Var
  i: Integer;
  d1,d2,d3: Double;
  s: String;
begin
  Initialize:=True;
  //These are the starting parameters
  b1:=Random(100);
  b2:=Random(100);
  b3:=Random(100);
  b4:=Random(100);
  //The following lines populate the array for Cosinus
  for i:=0 to 255 do
    begin
      ppIToD(d1,i);
      s:='3.14159';
      Str2Double(d3,s);
      dMul(d2,d1,d3);
      s:='128';
      Str2Double(d3,s);
      dDiv(d1,d2,d3);
      dCos(d2,d1);
      s:='64';
      Str2Double(d3,s);
      dMul(d1,d2,d3);
      s:='64';
      Str2Double(d3,s);
      dAdd(d2,d1,d3);
      d1:=d2;
      CosTbl[i]:=ppDToI(d1);
    end;
end;

begin
  if cmd=sysAppLaunchCmdNormalLaunch then
    if Initialize then begin
      InitScr;
      Main;
    end;
end.
