// Palm OS Constants

Const
 nilEvent              = 0;
 penDownEvent          = $0100;
 penUpEvent            = $0200;
 penMoveEvent          = $0300;
 keyDownEvent          = $0400;
 winEnterEvent         = $0500;
 winExitEvent          = $0600;
 ctlEnterEvent         = $0700;
 ctlExitEvent          = $0800;
 ctlSelectEvent        = $0900; 
 ctlRepeatEvent        = $0A00;
 lstEnterEvent         = $0B00;
 lstSelectEvent        = $0C00;
 lstExitEvent          = $0D00;
 popSelectEvent        = $0E00;
 fldEnterEvent         = $0F00;
 fldHeightChangedEvent = $1000;
 fldChangedEvent       = $1100;
 tblEnterEvent         = $1200;
 tblSelectEvent        = $1300;
 daySelectEvent        = $1400;
 menuEvent             = $1500;
 appStopEvent          = $1600;
 frmLoadEvent          = $1700;
 frmOpenEvent          = $1800;
 frmGotoEvent          = $1900;
 frmUpdateEvent        = $1A00;
 frmSaveEvent          = $1B00;
 frmCloseEvent         = $1C00;
 frmTitleEnterEvent    = $1D00;
 frmTitleSelectEvent   = $1E00;
 tblExitEvent          = $1F00;
 sclEnterEvent         = $2000;
 sclExitEvent          = $2100;
 sclRepeatEvent        = $2200;


// Palm OS Types Definitions

Type
 Byte      = 0..255;
 Int16     = -32768..32767;
 UInt16    = 0..65535;
 Int8      = -128..127;
 UInt8     = 0..255;
 Int32     = Integer;
 UInt32    = 0..MaxInt;
 WChar     = UInt16;
 StringPtr = ^String;
 LocalID   = UInt32;
 Err       = UInt16;
 Coord     = UInt16;

 Opaque    = record end;
 MemHandle = ^Opaque;
 ListPtr   = ^Opaque;
 FieldPtr  = ^Opaque;

 MemPtr    = Pointer;
 DmOpenRef = Pointer;

 PointType = record
              x,y:Coord;
             end;

 EventType = record
              eType:UInt16;
              penDown:Boolean;
              tapCount:UInt8;
              screenX,screenY:Int16;
              case integer of
               0: (datum:array[0..7] of UInt16);
               1: (PenUp:record start,finish:PointType end);
               2: (chr:WChar;keyCode,modifiers:UInt16);
               3: (FrmLoad:record FormID:UInt16 end);
               4: (ctlSelect:record controlID:UInt16;pControl:pointer;on:Boolean;reserved1:UInt8;value:UInt16 end);
               5: (menu:record itemID:UInt16 end);
               6: (ctlEnter:record controlID:UInt16;pControl:pointer end);
               7: (ctlRepeat:record controlID:UInt16;pControl:pointer;time:UInt32;value:UInt16 end);
               8: (sclRepeat:record scrollBarID:UInt16;pScrollbar:pointer;value,newValue:Int16;time:UInt32 end);
               9: (popSelect:record controlID:UInt16;pcontrol:pointer;listID:UInt16;plist:pointer;selection,priorselection:Int16 end);
               10:(frmTitleEnter:record formID:UInt16 end);
              end;

 AbsRectType = record
                left,top,right,bottom:Coord;
               end;

 FrameType = UInt16;
              
 RectangleType = record
                  topLeft,extent:PointType;
                 end;

 BitmapType = record end;
 BitmapPtr  = ^BitmapType;

 RGBColorType = record
                 index,r,g,b:UInt8;
                end;

 IndexedColorType = UInt8;

 FrameBitsType = record
                  bits,word:UInt16;
                 end;

 WinHandle  = ^WindowType;
 WindowType = record end;

 FormObjListType = record end;

 FormType = record end;
 FormPtr  = ^FormType;

 MenuBarPtr  = ^MenuBarType;
 MenuBarType = record end;

 ControlPtr  = ^ControlType;
 ControlType = record end;

 DmSearchStateType = record
                      info:array[0..7] of UInt32;
                     end;

 clipboardFormatType=(
  clipboardText,
  clipboardInk,
  clipboardBitmap);

 FontID=UInt8;

 UnderlineModeType=(
  noUnderline,
  grayUnderline,
  solidUnderline,
  colorUnderline);

 WinScreenModeOperation=(
  winScreenModeGetDefaults,
  winScreenModeGet,
  winScreenModeSetToDefaults,
  winScreenModeSet,
  winScreenModeGetSupportedDepths,
  winScreenModeGetSupportsColor);

 WinDirectionType=(
  WinUp,
  WinDown,
  WinLeft,
  WinRight);
  
 WinDrawOperation=(
  winPaint,
  winErase,
  winMask,
  winInvert,
  winOverlay,
  winPaintInverse,
  winSwap
 );

 ScrollBarType = opaque;
 ScrollBarPtr  = ^ScrollBarType;

 
// Palm OS API calls

function  BmpCreate(width,height:Coord;depth:UInt8;colortableP:pointer;var error:UInt16):BitmapPtr; SysTrap $3DD;
function  BmpGetBits(w:pointer):pointer; SysTrap $376;

procedure ClipboardAddItem(format:ClipboardFormatType;var ptr;length:UInt16); SysTrap $10A;
function  ClipboardGetItem(format:ClipboardFormatType;var length:UInt16):MemHandle;SysTrap $10C;

function  CtlGetLabel(const controlP: ControlPtr):StringPtr; SysTrap $113;
function  CtlGetValue(controlP:ControlPtr):UInt16;SysTrap $111;
procedure CtlSetLabel(controlP:ControlPtr;const newLabel:StringPtr); SysTrap $114;
procedure CtlSetValue(ControlP:ControlPtr;newValue:Int16); SysTrap $112;

function  DmCloseDatabase(dbP:DmOpenRef):UInt16; SysTrap $04A;
function  DmCreateDatabase(cardNo:UInt16;const name:String;c,t:UInt32;resDB:Boolean):UInt16; SysTrap $041;
function  DmDatabaseInfo(cardNo:UInt16;dbID:LocalID;var name:String;var attributes,version:UInt16;var crDate,modDate,bkpUpDate,modNum:UInt32;var appInfoID,sortInfoID:LocalID;var t,c:UInt32):UInt16; SysTrap $046;
function  DmDeleteDatabase(cardNo:UInt16;dbID:LocalID):UInt16; SysTrap $042;
function  DmFindDatabase(cardNo:UInt16;const name:String):LocalID; SysTrap $045;
function  DmFindResource(dbP:DmOpenRef;ResType:UInt32;ResID:UInt16;resH:MemHandle):UInt16; SysTrap $065;
function  DmGetDatabase (cardNo: UInt16; index: UInt16): LocalID; SysTrap $044;
function  DmGetNextDatabaseByTypeCreator(newSearch:Boolean;var stateInfo:DmSearchStateType;t,c:UInt32;onlyLastVers:Boolean;var cardNo:UInt16;var dbIDP:LocalID):UInt16; SysTrap $078;
function  DmGetRecord(dbP:DmOpenRef;index:UInt16):MemHandle; SysTrap $05C;
function  DmGetResource(typ:UInt32;resID:UInt16):Memhandle; SysTrap $05F;
function  DmGetResourceIndex(dbP:DmOpenRef;index:UInt16):MemHandle; SysTrap $06E;
function  DmGet1Resource(resType: UInt32;resID: UInt16): MemHandle; SysTrap $060;
function  DmNewRecord(dbP:DmOpenRef;var at:UInt16;size:UInt32):MemHandle; SysTrap $055;
function  DmNewResource(dbP:DmOpenRef;ResType:UInt32;ResID:UInt16;size:UInt32):MemHandle; SysTrap $06C;
function  DmNumDatabase(cardNo:UInt16):UInt16;SysTrap $043;
function  DmNumRecords (dbP: DmOpenRef): UInt16; SysTrap $04F;
function  DmNumRecordsInCategory(dbP:DmOpenRef;category:UInt16):UInt16; SysTrap $071;
function  DmNumResources(dbP:DmOpenRef):UInt16; SysTrap $067;
function  DmOpenDatabase(cardNo:UInt16;dbID:LocalID;mode:UInt16):DmOpenRef; SysTrap $049;
function  DmOpenDatabaseByTypeCreator(typo,creator:UInt32;mode:UInt16):DmOpenRef; SysTrap $075;
function  DmOpenDatabaseInfo(dbP:DmOpenRef;var dbIDP:LocalID;var openCountP,modeP,cardNoP:UInt16; var resDBP: Boolean):Err; SysTrap $04C;
function  DmQueryRecord(dbP:DmOpenRef;index:UInt16):MemHandle; SysTrap $05B;
function  DmReleaseRecord(dbP:DmOpenRef;index:UInt16;dirty:Boolean):UInt16; SysTrap $05E;
function  DmReleaseResource(resourceH:MemHandle):Err; SysTrap $061;
function  DmRemoveResource(dbP:DmOpenRef;index:UInt16):Err; SysTrap $06D;
function  DmResourceInfo(dbP:DmOpenRef;index:UInt16;var ResTypeP:UInt32;var ResID:UInt16;var chunkLocalIDP:LocalID):Err; SysTrap $068;
function  DmSet(rec:Pointer;offset:UInt32;bytes:UInt32;value:UInt8):UInt16; SysTrap $07E;
function  DmWrite(recordP:Pointer;offset:UInt32;srcP:Pointer;bytes:UInt32):UInt16; SysTrap $076;

procedure EvtGetEvent(var event:EventType;timeout:UInt32); SysTrap $11D;

procedure FldDrawField(fldP:FieldPtr); SysTrap $135;
procedure FldGetAttributes(fldP:pointer;var attrP:UInt16); SysTrap $14F;
function  FldGetTextHandle(fldP:FieldPtr):MemHandle;SysTrap $153;
function  FldGetTextLength(fldP:FieldPtr):UInt16;SysTrap $14B;
function  FldGetTextPtr(fldP:FieldPtr):StringPtr; SysTrap $139;
procedure FldSetAttributes(fldP:pointer; var attrP:UInt16); SysTrap $150;
procedure FldSetSelection(fldP:FieldPtr;startPosition,endPosition:UInt16);SysTrap $142;
procedure FldSetTextHandle(fldP:FieldPtr;textHandle:MemHandle);SysTrap $158;

function  FntCharsWidth(const Chars:String;Len:Int16):Int16; SysTrap $16B;
function  FntDefineFont(font:FontID;fontP:pointer):UInt16;SysTrap $321;
function  FntSetFont(font:FontID):FontID;SysTrap $164;

function  FrmAlert(alertId:UInt16):UInt16; SysTrap $192;
procedure FrmCloseAllForms;SysTrap $1A1;
procedure FrmCopyLabel(FormP:FormPtr;labelID:UInt16;Const newLabel:String); SysTrap $18C;
function  FrmCustomAlert(alertId:UInt16;const s1,s2,s3:String):UInt16; SysTrap $194;
procedure FrmDeleteForm(FormP:FormPtr); SysTrap $170;
function  FrmDispatchEvent(var event:EventType):Boolean; SysTrap $1A0;
function  FrmDoDialog(formP:FormPtr):UInt16; SysTrap $193;
procedure FrmDrawForm(FormP:FormPtr); SysTrap $171;
procedure FrmEraseForm(FormP:FormPtr); SysTrap $172;
function  FrmGetActiveForm:FormPtr; SysTrap $173;
function  FrmGetFocus(formP:FormPtr):UInt16;SysTrap $178;
function  FrmGetFormId(const formP:FormPtr):UInt16; SysTrap $17D;
function  FrmGetFormPtr(formId:UInt16):FormPtr; SysTrap $17E;
function  FrmGetLabel(formP:FormPtr;labelID:UInt16):StringPtr; SysTrap $18E;
procedure FrmGetObjectBounds(formP:FormPtr;objIndex:UInt16;var r:RectangleType); SysTrap $199;
function  FrmGetObjectIndex(formP:FormPtr;objID:UInt16):UInt16; SysTrap $180;
function  FrmGetObjectPtr(formP:FormPtr;objIndex:UInt16):pointer; SysTrap $183;
procedure FrmGotoForm(FormID:UInt16); SysTrap $19B;
function  FrmHandleEvent(formP:FormPtr;var event:EventType):Boolean; SysTrap $17A;
procedure FrmHelp(helpMsgId:UInt16); SysTrap $195;
procedure FrmHideObject(formP:FormPtr;objIndex:UInt16); SysTrap $184;
function  FrmInitForm(rscID:UInt16):FormPtr; SysTrap $16F;
procedure FrmPopupForm(formId:UInt16); SysTrap $19C;
procedure FrmReturnToForm(formId:UInt16); SysTrap $19E;
procedure FrmSetActiveForm(FormP:FormPtr); SysTrap $174;
procedure FrmSetControlGroupSelection(formP:FormPtr;groupNum:UInt8;controlID:UInt16); SysTrap $18B;
procedure FrmSetControlValue(formP:FormPtr;objIndex:UInt16;newValue:UInt16); SysTrap $189;
procedure FrmSetEventHandler(FormP:FormPtr;function handler(var event:Eventtype):Boolean); SysTrap $19F;
procedure FrmSetFocus(formP:FormPtr;fieldIndex:UInt16);SysTrap $179;
//procedure FrmSetObjectBounds(formP:FormPtr;objIndex:UInt16;r:RectangleType); SysTrap $303;
procedure FrmSetObjectPosition(FormP:FormPtr;objIndex:UInt16;x,y:Coord); SysTrap $187;
procedure FrmSetTitle(formP:FormPtr;const newTitle:string);SysTrap $191;
procedure FrmShowObject(formP:FormPtr;objIndex:UInt16); SysTrap $185;

function  FtrGet(creator:UInt32;featureNum:UInt16;var valueP:UInt32):Err; SysTrap $27B;

function  GetCharCaselessValue:pointer;SysTrap $296;

procedure InsPtEnable(enableIt : Boolean);SysTrap $1ab;
procedure InsPtSetHeight(Height:Int16); SysTrap $1AD;
procedure InsPtSetLocation(x,y:Int16); SysTrap $1a9;

function  KeyCurrentState:UInt32;SysTrap $2A2;

procedure LstDrawList(ListP:ListPtr);SysTrap $1B1;
procedure LstEraseList(ListP:ListPtr);SysTrap $1B2;
function  LstGetNumberOfItems(ListP:ListPtr):Int16;SysTrap $1BA;
function  LstGetSelection(ListP:ListPtr):Int16; SysTrap $1B3;
function  LstGetSelectionText(ListP:ListPtr;intemNum:Int16):StringPtr; SysTrap $1B4;
function  LstGetVisibleItems(ListP:ListPtr):Int16;SysTrap $2FF;
procedure LstMakeItemVisible(ListP:ListPtr;itemNum:Int16); SysTrap $0B9;
function  LstScrollList(ListP:ListPtr;direction:WinDirectionType;itemCount:Int16):Boolean;SysTrap $2FB;
procedure LstSetListChoices(listP:ListPtr;itemsText:Pointer;numItems:Int16); SysTrap $1B8;
procedure LstSetSelection(ListP:ListPtr;itemNum:Int16); SysTrap $1B7;
procedure LstSetTopItem(ListP:ListPtr;itemNum:Int16);SysTrap $2B5;

function  MemHandleFree(h: MemHandle):UInt16; SysTrap $02B;
function  MemHandleLock(h:MemHandle):MemPtr; SysTrap $021;
function  MemHandleNew(size:UInt32):MemHandle;SysTrap $01E;
function  MemHandleSize(h:MemHandle):UInt32; SysTrap $02D;
function  MemHandleUnlock(h:MemHandle):UInt16; SysTrap $022;
procedure MemMove(var dst,s;numBytes:integer); SysTrap $026;
function  MemPtrFree(p:MemPtr):UInt16; SysTrap $012;
function  MemPtrNew(size:UInt32):MemPtr; SysTrap $013;
function  MemPtrSetOwner(p:MemPtr;owner:UInt16):UInt16; SysTrap $01B;
procedure MemSet(var dst;numBytes:integer;value:UInt8); SysTrap $027;

procedure MenuEraseStatus(menuP:Pointer); SysTrap $1C1;
function  MenuHandleEvent(menuP:Pointer;var event:EventType;var error:UInt16):Boolean; SysTrap $1BF;

function  PrefGetAppPreferences(creator:UInt32;id:UInt16;var prefs;var prefsSize:UInt16;saved:Boolean):Int16; SysTrap $2D3;
procedure PrefSetAppPreferences(creator:UInt32;id,version:UInt16;var prefs;prefsSize:UInt16;saved:Boolean); SysTrap $2D4;

function  RctPtInRectangle(x,y:Coord;var r:RectangleType):Boolean; SysTrap $1C8;
procedure RctSetRectangle(var r:RectangleType;left,top,width,height:Coord); SysTrap $1C4;

procedure SclSetScrollBar(barP:ScrollBarPtr;value,min,max,pageSize:Int16);SysTrap $2B6;

procedure SndPlaySystemSound(beepID:Int8); SysTrap $234;

function  StrAToI(const S:string):integer; SysTrap $0ce;
function  StrCaselessCompare(const s1,s2:string):Int16; SysTrap $0CA;
procedure StrCopy(dst:MemPtr;const src:String); SysTrap $0C5;
procedure StrIToA(var S:string;N:integer); SysTrap $0C9;
function  StrIToH(var s:String;i:UInt32):StringPtr; SysTrap $0CB;
function  StrStr(const str,token:String):Pointer; SysTrap $0CD;

function  SysCurAppDatabase(var cardNo:UInt16;var dbID:LocalID):UInt16; SysTrap $0AC;
function  SysFormPointerArrayToStrings(c:Pointer;StringCount:Int16):MemHandle; SysTrap $0C1;
function  SysHandleEvent(var event:EventType):Boolean; SysTrap $0A9;
function  SysTaskDelay(delay:Int32):Err; SysTrap $0A0;
function  SysTicksPerSecond:UInt16; SysTrap $2E9;
function  SysUIAppSwitch(cardNo:UInt16;dbID:LocalID;cmd:UInt16;cmdPBP:Pointer):UInt16; SysTrap $0A7;

function  TimGetSeconds:UInt32; SysTrap $0F5;
function  TimGetTicks:UInt32; SysTrap $0F7;

function  WinCreateBitmapWindow(bitmapP:BitmapPtr;var error:UInt16):WinHandle; SysTrap $3E4;
procedure WinDeleteWindow(win:WinHandle;eraseIt:boolean); SysTrap $1F8;
procedure WinDrawBitmap(bitmapP:BitmapPtr;x,y:Coord); SysTrap $226;
procedure WinDrawChars(const chars : string;len:Int16;x,y:Coord); SysTrap $220;
procedure WinDrawChar(theChar : WChar; x,y:Coord); SysTrap $350;
procedure WinDrawLine(x1,y1,x2,y2:Coord); SysTrap $213;
procedure WinDrawRectangle(var r:RectangleType;cornerDiam:UInt16); SysTrap $218;
procedure WinDrawRectangleFrame(frame:FrameType;var rP:RectangleType); SysTrap $21B;
procedure WinEraseLine(x1,y1,x2,y2:Coord); SysTrap $215;
procedure WinEraseRectangle(var r:RectangleType;cornerDiam:UInt16); SysTrap $219;
function  WinGetBitmap(w:pointer):pointer; SysTrap $3a2;
function  WinGetDisplayWindow:pointer; SysTrap $200;
function  WinRGBToIndex(var rgbP:RGBColorType):IndexedColorType; SysTrap $39E;
function  WinScreenMode(operation:WinScreenModeOperation;var widthP,heightP,depthP:UInt32;var enableColorP:Boolean):Err; SysTrap $33E;
function  WinSetDrawWindow(win:WinHandle):WinHandle; SysTrap $1FD;
function  WinSetForeColor(foreColor:IndexedColorType):IndexedColorType; SysTrap $398;
function  WinSetBackColor(foreColor:IndexedColorType):IndexedColorType; SysTrap $399;
function  WinSetTextColor(foreColor:IndexedColorType):IndexedColorType; SysTrap $39A;
function  WinSetDrawMode(newMode:WinDrawOperation):WinDrawOperation; SysTrap $397;


// Utilities

function Swap32(n:UInt32):UInt32;
begin
  n:=(n shl 16)+(n shr 16);
  Swap32:=((n shl 8) and $ff00ff00)+((n shr 8) and $00ff00ff);
end;

function Swap16(n:UInt16):UInt16;
begin
  Swap16:=(n shl 8)+(n shr 8);
end;

